using UnityEngine;
using System.Collections;

public class BoardScript : MonoBehaviour {

    // standard size for triangular cell:
    // 2/sqrt(3), giving triangle height of 1
    const float SIDE_LENGTH = 1.1547f;

    public ClockScript Clock;   // Clock object
    public UIScript UI;         // Overlay UI

    public int NumRows = 5;     // Number of rows in the game board

    public GameObject PegPrefab;
    public GameObject HolePrefab;

    public Transform HoldPosition;  // Position for the selected peg

    private Transform _background;  // Quad containing the board texture

    public bool hasControl = true;  // Used to allow another object to handle a mouse click

    private PegScript _selectedPeg = null;  // Currently selected peg
    public PegScript selectedPeg {
        get { return _selectedPeg; }
        set {
            _selectedPeg = value;
            if (_selectedPeg == null) {
                // Recheck victory conditions whenever a peg is released
                CheckBoardState();
            }
        }
    }

    void Awake() {
        _background = transform.GetChild(0);
    }
	
	void LateUpdate () {
        // if something else uses the click, hasControl = false for this frame
        if (hasControl) {
            if (Input.GetMouseButtonDown(0) && (selectedPeg != null)) {
                selectedPeg.ResetToHole();
                selectedPeg = null;
            }
        } else {
            // regain control for the next frame
            hasControl = true;
        }
	}

    // Build board and begin
    public void StartGame() {
        ResetGameBoard();
        InitGameBoard();
        Clock.StartClock();
    }

    void InitGameBoard() {
        Vector3[] pegPositions = GetHolePositions();
        Point currentPoint = new Point(0, 0);

        // from the topmost point (0, 0):
        //  - add a hole to each point
        //  - add a peg to all but (0, 0)
        for (int i = 0; i < pegPositions.Length; i++) {
            // create hole and add child it to the board
            GameObject newHole = Instantiate(HolePrefab, pegPositions[i], Quaternion.identity) as GameObject;
            newHole.transform.SetParent(this.transform, true);
            HoleScript holeScript = newHole.GetComponent<HoleScript>();
            holeScript.point = currentPoint;

            // don't peg the first hole !
            if (i == 0) {
                currentPoint.y += 1;
                continue;
            }

            // all but first hole: add a peg
            GameObject newPeg = Instantiate(PegPrefab, pegPositions[i], Quaternion.identity) as GameObject;
            PegScript pegScript = newPeg.GetComponent<PegScript>();
            holeScript.peg = pegScript;
            pegScript.board = this;
            pegScript.point = currentPoint;

            // advance along the triangle
            if (currentPoint.x == currentPoint.y) {
                currentPoint.y++;
                currentPoint.x = 0;
            } else {
                currentPoint.x++;
            }
        }

        // set board scale to correspond with size of the board
        _background.localScale = new Vector3(NumRows * SIDE_LENGTH, NumRows, 1); // height = 1 * NumRows
    }

    void ResetGameBoard() {
        // kill all the spawns to allow the board to be rebuilt
        _selectedPeg = null;
        // child 0 is the background; start at 1
        for (int i = 1; i < transform.childCount; i++) {
            GameObject go = transform.GetChild(i).gameObject;
            PegScript peg = go.GetComponent<HoleScript>().peg;
            if (peg != null) {
                Destroy(peg.gameObject);
            }
            Destroy(go);
        }
    }

    public HoleScript GetHoleAt(Point p) {
        if (p.x < 0 || p.y < 0 || p.x > p.y || p.y >= NumRows) {
            // invalid hole requested
            return null;
        } else {
            // "1 + " to account for board background being child 0
            return transform.GetChild(1 + p.x + Triangle(p.y)).GetComponent<HoleScript>();
        }
    }

    // returns positions for the all the valid hole/peg spaces for this board size
    Vector3[] GetHolePositions() {
        int numPegs = Triangle(NumRows);
        Vector3[] pegPositions = new Vector3[numPegs];

        int currentPeg = 0;
        int row = 1; 
        int rowEnd = 1;

        float x = transform.position.x;
        float y = transform.position.y;
        float z = transform.position.z + (.5f * (NumRows - 1)); // set initial z to top of triangle

        while (currentPeg < pegPositions.Length) {
            pegPositions[currentPeg] = new Vector3(x, y, z);
            currentPeg++;
            if (currentPeg >= rowEnd) {
                // reset the position for the next row
                row++;
                rowEnd = Triangle(row);
                z -= 1f;
                // determine proper leftmost position for next row
                if (IsEven(row)) {
                    x = transform.position.x - ((.5f + ((row - 1) / 2)) * SIDE_LENGTH);
                } else {
                    x = transform.position.x - ((row / 2) * SIDE_LENGTH);
                }
            } else {
                // move right
                x += SIDE_LENGTH;
            }
        }

        return pegPositions;
    }

    // resets move validity on holes and checks for win/loss conditions
    void CheckBoardState() {
        int pegCount = 0;
        bool somethingCanMove = false;
        for (int i = 1; i < transform.childCount; i++) {
            HoleScript hole = transform.GetChild(i).GetComponent<HoleScript>();
            hole.validMove = false;
            if (hole.IsOccupied) {
                pegCount++;

                if (hole.peg.CanMove()) {
                    somethingCanMove = true;
                }
            }
        }

        if (pegCount == 1) {            // victory
            Clock.StopClock();
            StartCoroutine(UI.ShowReplayAfterDelay(true, 2f));
        } else if (!somethingCanMove) { // defeat
            Clock.StopClock();
            StartCoroutine(UI.ShowReplayAfterDelay(false, 3f));
        }
    }

    // math fxns

    // returns the Triangle Number for n, corresponding to total pegs in a triangle
    //      of n rows
    int Triangle(int n) {
        return (n * (n+1))/2;
    }

    bool IsEven(int n) {
        return ((n & 1) == 0);
    }

    // editor gizmos

    void OnDrawGizmos() {
        Gizmos.color = Color.red;
        Gizmos.DrawWireSphere(transform.position, .5f);

        Gizmos.color = Color.blue;
        foreach (Vector3 position in GetHolePositions()) {
            Gizmos.DrawWireSphere(position, .2f);
        }

        Gizmos.color = Color.white;
        Gizmos.DrawWireSphere(HoldPosition.position, .2f);
    }

    void OnDrawGizmosSelected() {
        Gizmos.color = Color.magenta;
        Gizmos.DrawWireSphere(transform.position, .5f);

        Gizmos.color = Color.cyan;
        foreach (Vector3 position in GetHolePositions()) {
            Gizmos.DrawWireSphere(position, .2f);
        }

        Gizmos.color = Color.white;
        Gizmos.DrawWireSphere(HoldPosition.position, .2f);
    }
}
