using UnityEngine;
using System.Collections;

public class PegScript : MonoBehaviour {

    public float moveTime;  // amount of time it takes to move the peg to a new position

    public Point point;         // this hole's location (see Point.cs)
    public BoardScript board;

    public AudioClip grabSound;
    public AudioClip dropSound;

    private Light _light;       // overhead spotlight
    private AudioSource _audio;

    private bool _moving;           // peg moving into or out of selection position
    private float _startTime;       // start time of current motion (for use in lerps)
    private bool _disabled = false; // true only if this peg is being removed

    // postions & rotations peg is moving to & from
    private Vector3 _startPosition; 
    private Quaternion _startRotation;
    private Vector3 _targetPosition;
    private Quaternion _targetRotation;

    // the six positions that could potentially be moved to in relation to this peg
    private readonly Point[] _validMoveOffsets = {new Point(-2, -2), new Point(0, -2), new Point(-2, 0),
                                   new Point(2, 0), new Point(0, 2), new Point(2, 2)};

    void Awake() {
        // light is the first child
        _light = transform.GetChild(0).GetComponent<Light>();
        _audio = transform.GetComponent<AudioSource>();
    }
	
	// Update is called once per frame
	void Update () {
        if (_moving) {  //moving into or out of a hole
            float t = (Time.time - _startTime) / moveTime;
            if (t >= 1) {
                _moving = false;
                transform.position = _targetPosition;
                transform.rotation = _targetRotation;
            } else {
                transform.position = Vector3.Lerp(_startPosition, _targetPosition, t);
                transform.rotation = Quaternion.Lerp(_startRotation, _targetRotation, t);
            }
        } else if (_disabled) { // moving off the edge of the screen to be destroyed
            float t = (Time.time - _startTime) / (moveTime * 2);
            if (t >= 1) {
                Destroy(this.gameObject);
            } else {
                transform.position = Vector3.Lerp(_startPosition, _targetPosition, t);
                transform.rotation = Quaternion.Lerp(_startRotation, _targetRotation, t);
            }
        }
	}

    // checks the 6 offsets and returns true as soon as it finds an offset that:
    // - Is on the triangle board
    // - Does not have a peg in it
    // - Does have a peg in between this point and the offset (the jumpable peg)
    // Returns false if ANY of the conditions fail for ALL six offsets
    public bool CanMove() {
        if (_disabled) {
            return false;
        }
        // offset + point are the valid jump positions
        foreach (Point offset in _validMoveOffsets) {
            HoleScript hole = board.GetHoleAt(this.point + offset);
            if (hole != null && !hole.IsOccupied) {
                // check that there is a peg between the spaces
                hole = board.GetHoleAt(this.point + (offset / 2));
                if (hole.IsOccupied) {
                    return true;
                }
            }
        }
        return false;
    }

    // Go to each valid position and let the hole know it is a valid move
    // Uses same logic as CanMove()
    void MarkSpaces() {
        foreach (Point offset in _validMoveOffsets) {
            HoleScript hole = board.GetHoleAt(this.point + offset);
            if (hole != null && !hole.IsOccupied) {
                // check that there is a peg between the spaces
                HoleScript midhole = board.GetHoleAt(this.point + (offset / 2));
                if (midhole.IsOccupied) {
                    hole.validMove = true;
                }
            }
        }
    }

    // Light up any peg with valid moves when it's moused over
    // Sublight it's potential jump locations (uses same logic as CanMove())
    void OnMouseEnter() {
        if (CanMove()) {
            // light it up
            _light.gameObject.SetActive(true);
            foreach (Point offset in _validMoveOffsets) {
                HoleScript hole = board.GetHoleAt(this.point + offset);
                if (hole != null && !hole.IsOccupied) {
                    // check that there is a peg between the spaces
                    HoleScript midhole = board.GetHoleAt(this.point + (offset / 2));
                    if (midhole.IsOccupied) {
                        hole.SetSublight(true);
                    }
                }
            }
        }
    }

    // Turn out the bright lights
    void OnMouseExit() {
        _light.gameObject.SetActive(false);
        foreach (Point offset in _validMoveOffsets) {
            HoleScript hole = board.GetHoleAt(this.point + offset);
            if (hole != null) {
                hole.SetSublight(false);
            }
        }
    }

    // If peg is valid && no other peg is selected, select peg & let board know it handled the click
    // (board.hasControl = false)
    void OnMouseDown() {
        if (board.selectedPeg == null && CanMove()) {
            board.selectedPeg = this;
            board.hasControl = false;
            MarkSpaces();
            MoveTo(board.HoldPosition.position, board.HoldPosition.rotation);
            _audio.PlayOneShot(grabSound);
        }
    }

    // Move peg to its currently assigned hole position && unselect it
    public void ResetToHole() {
        MoveTo(board.GetHoleAt(point).transform.position, Quaternion.identity);
        board.selectedPeg = null;
        _audio.PlayOneShot(dropSound);
    }

    // Execute jump move (only called from a hole with a valid jump position for the selected peg)
    public void JumpTo(HoleScript hole) {
        // remove self from old hole
        board.GetHoleAt(this.point).peg = null;

        // remove middle peg
        Point midpoint = this.point + ((hole.point - this.point) / 2);
        board.GetHoleAt(midpoint).peg.Remove();

        // assign & move to new hole
        point = hole.point;
        hole.peg = this;
        ResetToHole();
    }

    void MoveTo(Vector3 position, Quaternion rotation){
        _startTime = Time.time;
        _startPosition = transform.position;
        _startRotation = transform.rotation;

        _targetPosition = position;
        _targetRotation = rotation;
        _moving = true;
    }

    // Move the peg off to the right side of the board (it gets destroyed in the _disabled update loop)
    public void Remove() {
        board.GetHoleAt(this.point).peg = null;
        _startTime = Time.time;
        _startPosition = transform.position;
        _startRotation = transform.rotation;

        _targetPosition = new Vector3(5f, 3f, -2f);
        _targetRotation = Quaternion.identity;
        _targetRotation.SetLookRotation(Vector3.up, Vector3.right);
        _disabled = true;
    }
}
