/**
 * Point struct represents a position on the triangle
 * - x is the index from the left in a given row
 * - y is the index of the row from the top
 * Ex:
 *             (0,0)
 *          (0,1) (1,1)
 *       (0,2) (1,2) (2,2)
 *    (0,3) (1,3) (2,3) (3,3)
 * (0,4) (1,4) (2,4) (3,4) (4,4)
 * 
 * This scheme was chosen so that (0,0) is always the top of a triangle of any given size
 * Notes:
 * - For a valid triangle in this system, (0 <= x <= y) && (0 <= y <= rows-1)
 * - Math operations have been defined on the Points to use them to calculate offsets
 *      (e.g. the point to the left of a point P is represented by P + Point(-1, 0) )
 **/

public struct Point {
    public int x;
    public int y;

    public Point(int x, int y) {
        this.x = x;
        this.y = y;
    }

    public override bool Equals(object obj) {
        if (!(obj is Point)) {
            return false;
        }

        return this.Equals((Point)obj);
    }

    public bool Equals(Point p) {
        return (x == p.x) && (y == p.y);
    }

    public static bool operator ==(Point p1, Point p2) {
        return p1.Equals(p2);
    }

    public static bool operator !=(Point p1, Point p2) {
        return !(p1 == p2);
    }

    public static Point operator +(Point p1, Point p2) {
        return new Point(p1.x + p2.x, p1.y + p2.y);
    }

    public static Point operator -(Point p1, Point p2) {
        return new Point(p1.x - p2.x, p1.y - p2.y);
    }

    public static Point operator /(Point p, int f) {
        return new Point(p.x / f, p.y / f);
    }

    public override int GetHashCode() {
        return x ^ y;
    }

    public override string ToString() {
        return "Point: (" + x + ", " + y + ")";
    }
}
