using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class UIScript : MonoBehaviour {

    public float fadeInTime;    // amount of time to lerp into/out of the main title
    public float flashInterval; // speed at which to flash the Click To Begin dialogue

    public Image underlay;          // Dark image to block the board/clock pre-initialisation
    public Text nameText;           // Main title text
    public Text flashingText;       // Click To Begin dialogue
    public GameObject replayPanel;  // Panel containing buttons to reset the board

    public AudioClip startSound;
    public AudioClip winSound;
    public AudioClip loseSound;

    public BoardScript board;

    private AudioSource _audio;
    private float _lastTimeEvent;
    private bool _fadingIntoGame = false;   // triggered after player clicks on the title screen

    void Awake() {
        _audio = GetComponent<AudioSource>();
        flashingText.gameObject.SetActive(false);
        board.gameObject.SetActive(false);
        replayPanel.SetActive(false);
    }

    void Start() {
        // set name text to transparent white so it can fade in
        nameText.color = new Color(1f, 1f, 1f, 0f);
        _lastTimeEvent = Time.time + fadeInTime;
	}
	
	void Update () {
        if (_fadingIntoGame) {
            float t = (Time.time - _lastTimeEvent) / fadeInTime;

            if (t >= 1f) {
                // completely faded in, kill underlay
                underlay.gameObject.SetActive(false);
            } else {
                underlay.color = Color.Lerp(Color.black, Color.clear, t);
            }
        } else {
            if (Input.GetMouseButtonDown(0)) {
                if (!flashingText.gameObject.activeSelf) {
                    //player clicked during title fade in
                    //skip rest of fade and start flash
                    nameText.color = Color.white;
                    _lastTimeEvent = Time.time;
                    flashingText.gameObject.SetActive(true);
                    flashingText.color = Color.clear;
                } else {
                    //clicked during flashing cycle
                    //start the game
                    board.gameObject.SetActive(true);
                    board.StartGame();
                    _fadingIntoGame = true;
                    _lastTimeEvent = Time.time;
                    nameText.gameObject.SetActive(false);
                    flashingText.gameObject.SetActive(false);
                    _audio.PlayOneShot(startSound);
                }
            }

            if (flashingText.gameObject.activeSelf) {
                //flashing click prompt (done fading in)
                //  alternate between white/clear text until clicked
                if (Time.time >= _lastTimeEvent + flashInterval) {
                    if (flashingText.color == Color.clear) {
                        flashingText.color = Color.white;
                    } else {
                        flashingText.color = Color.clear;
                    }
                    _lastTimeEvent = Time.time;
                }

            } else {
                // fading in title text
                float t = (Time.time - _lastTimeEvent) / fadeInTime;

                if (t >= 1f) {
                    // completely faded in, start flashing click text
                    nameText.color = Color.white;
                    _lastTimeEvent = Time.time;
                    flashingText.gameObject.SetActive(true);
                    flashingText.color = Color.clear;
                } else {
                    nameText.color = Color.Lerp(new Color(1f, 1f, 1f, 0f), Color.white, t);
                }
            }
        }
	}

    // called by board to bring up replay dialogue
    public IEnumerator ShowReplayAfterDelay(bool won, float delay) {
        Text rp_text = replayPanel.transform.GetChild(0).GetComponent<Text>();
        if (won) {
            rp_text.text = "You Win!";
            _audio.PlayOneShot(winSound);
        } else {
            rp_text.text = "You lost :(";
            _audio.PlayOneShot(loseSound);
        }
        rp_text.text += "\nPlay again? Choose difficulty:";
        yield return new WaitForSeconds(delay);
        replayPanel.SetActive(true);
    }

    // method attached to replay buttons to resize board
    public void NewGame(int numRows) {
        board.NumRows = numRows;
        board.StartGame();
        replayPanel.SetActive(false);
        _audio.PlayOneShot(startSound);
    }
}
